from optparse import OptionParser

from parsley.config import *
from parsley.loggingex import *

from util import *

log = logging.getLogger("cumin.config")

class CuminConfig(Config):
    def __init__(self):
        super(CuminConfig, self).__init__()

        hdef = os.path.normpath("/usr/share/cumin")
        self.home = os.environ.get("CUMIN_HOME", hdef)

        if not os.path.isdir(self.home):
            raise Exception("Home path '%s' is not a directory")

    def get_home(self):
        return self.home

    def parse(self):
        paths = list()

        paths.append(os.path.join(os.sep, "etc", "cumin", "cumin.conf"))
        paths.append(os.path.join(self.home, "etc", "cumin.conf"))
        paths.append(os.path.join(os.path.expanduser("~"), ".cumin.conf"))

        return self.parse_files(paths)

class CuminWebConfig(CuminConfig):
    def __init__(self, section_name):
        super(CuminWebConfig, self).__init__()

        self.create_section(section_name)

    def create_section(self, name):
        web = CuminConfigSection(self, name)
        web.log_file.default = os.path.join(self.home, "log", name + ".log")

        param = ConfigParameter(web, "update-interval", int)
        param.default = 10

        param = ConfigParameter(web, "host", str)
        param.default = "localhost"

        param = ConfigParameter(web, "port", int)
        param.default = 45672

        param = ConfigParameter(web, "operator-email", str)

        param = ConfigParameter(web, "user", str)

        param = ConfigParameter(web, "request-memory",    int)
        param.default = 512 # MB
        param = ConfigParameter(web, "request-memory-vm", int)
        param.default = 512 # MB
        param = ConfigParameter(web, "request-disk",      int)
        param.default = 1024 #MB
        param = ConfigParameter(web, "request-disk-vm",   int)
        param.default = 5 * 1024 #MB

class CuminDataConfig(CuminConfig):
    def __init__(self, section_name):
        super(CuminDataConfig, self).__init__()

        self.create_section(section_name)

    def create_section(self, name):
        data = CuminConfigSection(self, name)
        data.log_file.default = os.path.join(self.home, "log", name + ".log")
        
        param = ConfigParameter(data, "packages", str)

        param = ConfigParameter(data, "expire-interval", int)
        param.default = 60 * 60 # 1 hour

        param = ConfigParameter(data, "expire-threshold", int)
        param.default = 24 * 60 * 60 # 1 day

        param = ConfigParameter(data, "vacuum-interval", int)
        param.default = 60 * 60 # 1 hour

class CuminConfigSection(ConfigSection):
    def __init__(self, config, name):
        super(CuminConfigSection, self).__init__(config, name)

        param = ConfigParameter(self, "database", str)
        param.default = "dbname=cumin user=cumin host=localhost"

        param = ConfigParameter(self, "brokers", str)
        param.default = "amqp://localhost"

        self.log_file = ConfigParameter(self, "log-file", str)

        param = ConfigParameter(self, "log-level", str)
        param.default = "info"

        param = ConfigParameter(self, "debug", bool)
        param.default = False

class CuminMultiOptionParser(OptionParser,object):
    def __init__(self, default_section):
        OptionParser.__init__(self)

        self.add_option("--section", default=default_section)

        # We don't know defaults yet...          
        self.add_option("--database")
        self.add_option("--brokers")
        self.add_option("--log-file")
        self.add_option("--log-level")
        self.add_option("--debug", action="store_true")
        self.add_option("--init-only", action="store_true")

    def _apply_default(self, section, name):
        try:
            val = getattr(section, name)
            self.set_default(name, val)
        except:
            pass

    def set_defaults_from_section(self, section):
        # It is possible that a section does not contain an updated
        # default value for a particular option.  Skip exception.
        self._apply_default(section, "database")
        self._apply_default(section, "brokers")
        self._apply_default(section, "log_file")
        self._apply_default(section, "log_level")
        self._apply_default(section, "debug")

class CuminWebOptionParser(CuminMultiOptionParser):
    def __init__(self, default_section="web"):
        super(CuminWebOptionParser, self).__init__(default_section)

        # Add additional parameters for web
        self.add_option("--host")
        self.add_option("--port", type=int)

    def set_defaults_from_section(self, section):
        super(CuminWebOptionParser, self).set_defaults_from_section(section)

        # For web, these options also have defaults in config files
        self._apply_default(section, "host")
        self._apply_default(section, "port")

class CuminDataOptionParser(CuminMultiOptionParser):
    def __init__(self, default_section="data"):
        super(CuminDataOptionParser, self).__init__(default_section)

        # Add additional parameters for data
        self.add_option("--print-stats", action="store_true")
        self.add_option("--print-events", type="int", default=0, metavar="LEVEL")
        
def setup_logging(values):
    modules = ("cumin", "mint", "parsley", "rosemary", "wooly")

    for name in modules:
        enable_logging(name, values.log_level, values.log_file)

    if values.debug:
        for name in modules:
            enable_logging(name, "debug", sys.stderr)

def get_configuration(parser, ConfigType):
    # First parse should return a "section" value
    opts, args = parser.parse_args()
    if hasattr(opts, "section"):
        # Create config object with the given section and parse config files
        config = ConfigType(opts.section)
        values = config.parse()
        if hasattr(values, opts.section):
            # Reset defaults and parse again
            values = getattr(values, opts.section)
            parser.set_defaults_from_section(values)
            opts, args = parser.parse_args()
        else:
            raise Exception("Section ["+opts.section+"] not present in configuration files.")
    else:
        raise Exception("No '--section' value available as option or default")
    return config, values, opts, args
